@@ -2,12 +2,12 @@
 #
 #	Implementation of doctools objects for Tcl.
 #
-# Copyright (c) 2003-2007 Andreas Kupries <andreas_kupries@sourceforge.net>
+# Copyright (c) 2003-2008 Andreas Kupries <andreas_kupries@sourceforge.net>
 #
 # See the file "license.terms" for information on usage and redistribution
 # of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 # 
-# RCS: @(#) $Id: doctools.tcl,v 1.22 2007/08/20 20:11:59 andreas_kupries Exp $
+# RCS: @(#) $Id: doctools.tcl,v 1.31 2008/12/01 23:49:01 andreas_kupries Exp $
 
 package require Tcl 8.2
 package require textutil::expander
@@ -129,6 +129,7 @@ proc ::doctools::help {} {
 	    * example       - example block\n\
 	    * example_begin - begin example\n\
 	    * example_end   - end of example\n\
+	    * category      - category declaration\n\
 	    * see_also      - cross reference declaration\n\
 	    * keywords      - keyword declaration\n\
 	    * nl            - paragraph break in list items\n\
@@ -275,19 +276,19 @@ proc ::doctools::_cget {name option} {
 #	The value of one particular option if called with a single argument.
 
 proc ::doctools::_configure {name args} {
-    upvar ::doctools::doctools${name}::format_ip  format_ip
-    upvar ::doctools::doctools${name}::chk_ip     chk_ip
-    upvar ::doctools::doctools${name}::expander   expander
-    upvar ::doctools::doctools${name}::passes     passes
+    upvar #0 ::doctools::doctools${name}::format_ip  format_ip
+    upvar #0 ::doctools::doctools${name}::chk_ip     chk_ip
+    upvar #0 ::doctools::doctools${name}::expander   expander
+    upvar #0 ::doctools::doctools${name}::passes     passes
 
     if {[llength $args] == 0} {
 	# Retrieve the current configuration.
 
-	upvar ::doctools::doctools${name}::file       file
-	upvar ::doctools::doctools${name}::module     module
-	upvar ::doctools::doctools${name}::format     format
-	upvar ::doctools::doctools${name}::copyright  copyright
-	upvar ::doctools::doctools${name}::deprecated deprecated
+	upvar #0 ::doctools::doctools${name}::file       file
+	upvar #0 ::doctools::doctools${name}::module     module
+	upvar #0 ::doctools::doctools${name}::format     format
+	upvar #0 ::doctools::doctools${name}::copyright  copyright
+	upvar #0 ::doctools::doctools${name}::deprecated deprecated
 
 	set     res [list]
 	lappend res -file       $file
@@ -302,23 +303,23 @@ proc ::doctools::_configure {name args} {
 
 	switch -exact -- [lindex $args 0] {
 	    -file {
-		upvar ::doctools::doctools${name}::file file
+		upvar #0 ::doctools::doctools${name}::file file
 		return $file
 	    }
 	    -module {
-		upvar ::doctools::doctools${name}::module module
+		upvar #0 ::doctools::doctools${name}::module module
 		return $module
 	    }
 	    -copyright {
-		upvar ::doctools::doctools${name}::copyright copyright
+		upvar #0 ::doctools::doctools${name}::copyright copyright
 		return $copyright
 	    }
 	    -format {
-		upvar ::doctools::doctools${name}::format format
+		upvar #0 ::doctools::doctools${name}::format format
 		return $format
 	    }
 	    -deprecated {
-		upvar ::doctools::doctools${name}::deprecated deprecated
+		upvar #0 ::doctools::doctools${name}::deprecated deprecated
 		return $deprecated
 	    }
 	    default {
@@ -337,22 +338,22 @@ proc ::doctools::_configure {name args} {
 	foreach {option value} $args {
 	    switch -exact -- $option {
 		-file {
-		    upvar ::doctools::doctools${name}::file file
+		    upvar #0 ::doctools::doctools${name}::file file
 		    set file $value
 		}
 		-module {
-		    upvar ::doctools::doctools${name}::module module
+		    upvar #0 ::doctools::doctools${name}::module module
 		    set module $value
 		}
 		-copyright {
-		    upvar ::doctools::doctools${name}::copyright copyright
+		    upvar #0 ::doctools::doctools${name}::copyright copyright
 		    set copyright $value
 		}
 		-format {
 		    if {[catch {
 			set fmtfile [LookupFormat $name $value]
 			SetupFormatter $name $fmtfile
-			upvar ::doctools::doctools${name}::format format
+			upvar #0 ::doctools::doctools${name}::format format
 			set format $value
 		    } msg]} {
 			return -code error "doctools::_configure: -format: $msg"
@@ -363,7 +364,7 @@ proc ::doctools::_configure {name args} {
 			return -code error \
 				"doctools::_configure: -deprecated expected a boolean, got \"$value\""
 		    }
-		    upvar ::doctools::doctools${name}::deprecated deprecated
+		    upvar #0 ::doctools::doctools${name}::deprecated deprecated
 		    set deprecated $value
 		}
 		default {
@@ -420,7 +421,7 @@ proc ::doctools::_destroy {name} {
 #	None.
 
 proc ::doctools::_map {name sfname afname} {
-    upvar ::doctools::doctools${name}::map map
+    upvar #0 ::doctools::doctools${name}::map map
     set map($sfname) $afname
     return
 }
@@ -438,17 +439,17 @@ proc ::doctools::_map {name sfname afname} {
 #	The conversion result.
 
 proc ::doctools::_format {name text} {
-    upvar ::doctools::doctools${name}::format format
+    upvar #0 ::doctools::doctools${name}::format format
     if {$format == ""} {
 	return -code error "$name: No format was specified"
     }
 
-    upvar ::doctools::doctools${name}::format_ip format_ip
-    upvar ::doctools::doctools${name}::chk_ip    chk_ip
-    upvar ::doctools::doctools${name}::ex_ok     ex_ok
-    upvar ::doctools::doctools${name}::expander  expander
-    upvar ::doctools::doctools${name}::passes    passes
-    upvar ::doctools::doctools${name}::msg       warnings
+    upvar #0 ::doctools::doctools${name}::format_ip format_ip
+    upvar #0 ::doctools::doctools${name}::chk_ip    chk_ip
+    upvar #0 ::doctools::doctools${name}::ex_ok     ex_ok
+    upvar #0 ::doctools::doctools${name}::expander  expander
+    upvar #0 ::doctools::doctools${name}::passes    passes
+    upvar #0 ::doctools::doctools${name}::msg       warnings
 
     if {!$ex_ok}       {SetupExpander  $name}
     if {$chk_ip == ""} {SetupChecker   $name}
@@ -471,7 +472,7 @@ proc ::doctools::_format {name text} {
 	    catch {$format_ip eval fmt_shutdown}
 	    return -code error "Could not initialize pass $n of engine"
 	}
-	$chk_ip eval ck_initialize
+	$chk_ip eval ck_initialize $n
 
 	if {[catch {set result [$expander expand $text]} msg]} {
 	    catch {$format_ip eval fmt_shutdown}
@@ -516,7 +517,7 @@ proc ::doctools::_search {name path} {
     if {![file isdirectory $path]} {return -code error "$name search: path is not a directory"}
     if {![file readable    $path]} {return -code error "$name search: path cannot be read"}
 
-    upvar ::doctools::doctools${name}::paths paths
+    upvar #0 ::doctools::doctools${name}::paths paths
     set paths [linsert $paths 0 $path]
     return
 }
@@ -532,7 +533,7 @@ proc ::doctools::_search {name path} {
 #	A list of warnings.
 
 proc ::doctools::_warnings {name} {
-    upvar ::doctools::doctools${name}::msg msg
+    upvar #0 ::doctools::doctools${name}::msg msg
     return $msg
 }
 
@@ -548,7 +549,7 @@ proc ::doctools::_warnings {name} {
 #	A list of parameter names
 
 proc ::doctools::_parameters {name} {
-    upvar ::doctools::doctools${name}::param param
+    upvar #0 ::doctools::doctools${name}::param param
     return $param
 }
 
@@ -565,7 +566,7 @@ proc ::doctools::_parameters {name} {
 #	None.
 
 proc ::doctools::_setparam {name param value} {
-    upvar ::doctools::doctools${name}::format_ip format_ip
+    upvar #0 ::doctools::doctools${name}::format_ip format_ip
 
     if {$format_ip == {}} {
 	return -code error \
@@ -601,7 +602,7 @@ proc ::doctools::LookupFormat {name format} {
 	return $format
     }
 
-    upvar ::doctools::doctools${name}::paths opaths
+    upvar #0 ::doctools::doctools${name}::paths opaths
     foreach path $opaths {
 	set f [file join $path fmt.$format]
 	if {[file exists $f]} {
@@ -649,6 +650,7 @@ proc ::doctools::SetupFormatter {name format} {
     $mpip invokehidden source [file join $here api.tcl]
     #$mpip eval [list source [file join $here api.tcl]]
     interp alias $mpip dt_source   {} ::doctools::Source  $mpip [file dirname $format]
+    interp alias $mpip dt_read     {} ::doctools::Read    $mpip [file dirname $format]
     interp alias $mpip dt_package  {} ::doctools::Package $mpip
     interp alias $mpip file        {} ::doctools::FileOp  $mpip
     interp alias $mpip puts_stderr {} ::puts stderr
@@ -694,12 +696,12 @@ proc ::doctools::SetupFormatter {name format} {
     # now invalid. It will be recreated during the
     # next call of 'format'.
 
-    upvar ::doctools::doctools${name}::formatfile formatfile
-    upvar ::doctools::doctools${name}::format_ip  format_ip
-    upvar ::doctools::doctools${name}::chk_ip     chk_ip
-    upvar ::doctools::doctools${name}::expander   expander
-    upvar ::doctools::doctools${name}::passes     xpasses
-    upvar ::doctools::doctools${name}::param      xparam
+    upvar #0 ::doctools::doctools${name}::formatfile formatfile
+    upvar #0 ::doctools::doctools${name}::format_ip  format_ip
+    upvar #0 ::doctools::doctools${name}::chk_ip     chk_ip
+    upvar #0 ::doctools::doctools${name}::expander   expander
+    upvar #0 ::doctools::doctools${name}::passes     xpasses
+    upvar #0 ::doctools::doctools${name}::param      xparam
 
     if {$chk_ip != {}}    {interp delete $chk_ip}
     if {$format_ip != {}} {interp delete $format_ip}
@@ -749,11 +751,11 @@ proc ::doctools::SetupChecker {name} {
 
     variable here
 
-    upvar ::doctools::doctools${name}::chk_ip    chk_ip
+    upvar #0 ::doctools::doctools${name}::chk_ip    chk_ip
     if {$chk_ip != ""} {return}
 
-    upvar ::doctools::doctools${name}::expander  expander
-    upvar ::doctools::doctools${name}::format_ip format_ip
+    upvar #0 ::doctools::doctools${name}::expander  expander
+    upvar #0 ::doctools::doctools${name}::format_ip format_ip
 
     set chk_ip [interp create] ; # interpreter hosting the formal format checker
 
@@ -787,7 +789,7 @@ proc ::doctools::SetupChecker {name} {
 	keywords nl arg cmd opt comment sectref syscmd method option
 	widget fun type package class var file uri usage term const
 	arg_def cmd_def opt_def tkoption_def emph strong plain_text
-	namespace subsection
+	namespace subsection category
     } {
 	interp alias $chk_ip fmt_$cmd $format_ip fmt_$cmd
     }
@@ -805,10 +807,10 @@ proc ::doctools::SetupChecker {name} {
 #	None.
 
 proc ::doctools::SetupExpander {name} {
-    upvar ::doctools::doctools${name}::ex_ok    ex_ok
+    upvar #0 ::doctools::doctools${name}::ex_ok    ex_ok
     if {$ex_ok} {return}
 
-    upvar ::doctools::doctools${name}::expander expander
+    upvar #0 ::doctools::doctools${name}::expander expander
     ::textutil::expander $expander
     $expander evalcmd [list ::doctools::Eval $name]
     $expander textcmd plain_text
@@ -828,7 +830,7 @@ proc ::doctools::SetupExpander {name} {
 #	None.
 
 proc ::doctools::SearchPaths {name} {
-    upvar ::doctools::doctools${name}::paths opaths
+    upvar #0 ::doctools::doctools${name}::paths opaths
     variable paths
 
     set p $opaths
@@ -848,7 +850,7 @@ proc ::doctools::SearchPaths {name} {
 #	None.
 
 proc ::doctools::Deprecated {name} {
-    upvar ::doctools::doctools${name}::deprecated deprecated
+    upvar #0 ::doctools::doctools${name}::deprecated deprecated
     return $deprecated
 }
 
@@ -879,7 +881,7 @@ proc ::doctools::FmtError {name text} {
 #	None.
 
 proc ::doctools::FmtWarning {name text} {
-    upvar ::doctools::doctools${name}::msg msg
+    upvar #0 ::doctools::doctools${name}::msg msg
     lappend msg $text
     return
 }
@@ -895,7 +897,7 @@ proc ::doctools::FmtWarning {name text} {
 #	List containing offset, line, column
 
 proc ::doctools::Where {name} {
-    upvar ::doctools::doctools${name}::expander expander
+    upvar #0 ::doctools::doctools${name}::expander expander
     return [$expander where]
 }
 
@@ -911,7 +913,7 @@ proc ::doctools::Where {name} {
 #	None.
 
 proc ::doctools::Eval {name macro} {
-    upvar ::doctools::doctools${name}::chk_ip chk_ip
+    upvar #0 ::doctools::doctools${name}::chk_ip chk_ip
 
     #puts stderr "\t\t$name [lindex [split $macro] 0]"
 
@@ -945,7 +947,7 @@ proc ::doctools::Eval {name macro} {
 #	None.
 
 proc ::doctools::ExpandInclude {name path} {
-    upvar ::doctools::doctools${name}::file file
+    upvar #0 ::doctools::doctools${name}::file file
 
     set ipath [file join [file dirname $file] $path]
     if {![file exists $ipath]} {
@@ -959,7 +961,7 @@ proc ::doctools::ExpandInclude {name path} {
     set    text [read $chan]
     close $chan
 
-    upvar ::doctools::doctools${name}::expander  expander
+    upvar #0 ::doctools::doctools${name}::expander  expander
 
     return [$expander expand $text]
 }
@@ -993,7 +995,7 @@ proc ::doctools::GetFile {name} {
 
     #puts stderr "GetFile $name"
 
-    upvar ::doctools::doctools${name}::file file
+    upvar #0 ::doctools::doctools${name}::file file
 
     #puts stderr "ok $file"
     return $file
@@ -1042,7 +1044,7 @@ proc ::doctools::FileCmd {cmd args} {
 #	Module information
 
 proc ::doctools::GetModule {name} {
-    upvar ::doctools::doctools${name}::module module
+    upvar #0 ::doctools::doctools${name}::module module
     return   $module
 }
 
@@ -1057,7 +1059,7 @@ proc ::doctools::GetModule {name} {
 #	Copyright information
 
 proc ::doctools::GetCopyright {name} {
-    upvar ::doctools::doctools${name}::copyright copyright
+    upvar #0 ::doctools::doctools${name}::copyright copyright
     return   $copyright
 }
 
@@ -1072,7 +1074,7 @@ proc ::doctools::GetCopyright {name} {
 #	Format information
 
 proc ::doctools::GetFormat {name} {
-    upvar ::doctools::doctools${name}::format format
+    upvar #0 ::doctools::doctools${name}::format format
     return $format
 }
 
@@ -1087,7 +1089,7 @@ proc ::doctools::GetFormat {name} {
 #	Boolean flag.
 
 proc ::doctools::ListLevel {name} {
-    upvar ::doctools::doctools${name}::chk_ip chk_ip
+    upvar #0 ::doctools::doctools${name}::chk_ip chk_ip
     return [$chk_ip eval LNest]
 }
 
@@ -1105,7 +1107,7 @@ proc ::doctools::ListLevel {name} {
 #	Actual name of the file.
 
 proc ::doctools::MapFile {name fname} {
-    upvar ::doctools::doctools${name}::map map
+    upvar #0 ::doctools::doctools${name}::map map
 
     #parray map
 
@@ -1134,6 +1136,11 @@ proc ::doctools::Source {ip path file} {
     return
 }
 
+proc ::doctools::Read {ip path file} {
+    #puts stderr "$ip (read $path $file)"
+
+    return [read [set f [open [file join $path [file tail $file]]]]][close $f]
+}
 
 proc ::doctools::Locate {p} {
     # @mdgen NODEP: doctools::__undefined__
@@ -1191,4 +1198,4 @@ namespace eval ::doctools {
     catch {search [file join $here                             mpformats]}
 }
 
-package provide doctools 1.3
+package provide doctools 1.4
