@@ -1,5 +1,5 @@
 if {![package vsatisfies [package provide Tcl] 8]} {return}
-package ifneeded log 1.2 [list source [file join $dir log.tcl]]
+package ifneeded log 1.2.1 [list source [file join $dir log.tcl]]
 
 if {![package vsatisfies [package provide Tcl] 8.2]} {return}
 package ifneeded logger           0.8 [list source [file join $dir logger.tcl]]
